/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import net.sf.cglib.proxy.Enhancer;
import net.sf.cglib.proxy.NoOp;

/**
 * @author "Pierre Vigneras"
 * @date Dec 5, 2007
 */
public final class ProxiedTests {
  private ProxiedTests() { }


  
  public static Test suite(TestSuite suiteToProxy) {
    TestSuite suite = new TestSuite(ProxiedTests.class.getName());
    try {
      Set<Class< ? extends TestCase>> testClasses = new HashSet<Class< ? extends TestCase>>();
      testClasses = addTests(testClasses, suiteToProxy);
      for (Class< ? extends TestCase> clazz : testClasses) {
        suite.addTestSuite(clazz);
      }
    } catch (Exception e) {
      e.printStackTrace();
      throw new RuntimeException("problem while building proxy test suite : " + e.getMessage());
    }

    return suite;
  }


  @SuppressWarnings("unchecked")
  public static Set<Class< ? extends TestCase>> addTests(Set<Class< ? extends TestCase>> tests,
      TestSuite testSuite) throws Exception {

    Enumeration<Test> enumeration = testSuite.tests();
    if (enumeration != null) {
      while (enumeration.hasMoreElements()) {
        Test test = enumeration.nextElement();
        Class < ? > testClass = test.getClass();
        if (TestSuite.class.isAssignableFrom(testClass)) {
          //it is a suite
          TestSuite suite = (TestSuite) test;
          tests = addTests(tests, suite);
        } else if (TestCase.class.isAssignableFrom(testClass)) {
          Enhancer e = new Enhancer();
          e.setSuperclass(testClass);
          e.setCallbackType(NoOp.class);
          Class<TestCase> newTestClass = e.createClass();
          tests.add(newTestClass);
        }
      }
    }
    return tests;
  }

}
