package org.ow2.bonita.activity.instantiation;

import java.net.URL;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.activity.instantiation.instantiator.JoinNumberConditionFail;
import org.ow2.bonita.activity.instantiation.instantiator.JoinNumberConditionPass;
import org.ow2.bonita.activity.instantiation.instantiator.TestOneJoinActivityInstantiator;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.BonitaException;

public class NewFeatureMultiInstTest extends APITestCase {
  

  // Fails because of subflow archiving
  public void testSubflowMultiInstantiation() throws BonitaException {
    //Test subflow with multi instantiation
    URL xpdlUrl = this.getClass().getResource("subflowMultiInstantiation.xpdl");
    String processId = "mainCondPass";
    Map<String, ProcessDefinition> processMap = getManagementAPI().deployBar(getBarContent(
        xpdlUrl, JoinNumberConditionPass.class, JoinNumberConditionFail.class, CheckExecutionHook.class));
    
    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processMap.get(processId).getUUID());
    checkExecutedManyTimes(instanceUUID, new String[]{"a2"}, 4);
    checkExecutedOnce(instanceUUID, new String[]{"a3", "a4"});
    
    processId = "mainCondFail";
    instanceUUID = getRuntimeAPI().instantiateProcess(processMap.get(processId).getUUID());
    checkExecutedManyTimes(instanceUUID, new String[]{"a2"}, 3);
    checkStopped(instanceUUID, new String[]{"a3"});
    
    try {
      getQueryRuntimeAPI().getActivityInstances(instanceUUID, "a4");
      fail("Activity : a3 must not be executed in this instance!");
    } catch (ActivityNotFoundException e) {
      //nothing, we expect an exception
    }
    getRuntimeAPI().deleteAllProcessInstances(processMap.get(processId).getUUID());
    
    getManagementAPI().undeploy(processMap.get(processId).getPackageDefinitionUUID());
  }
  
  
  // Fail because remaining instances are not canceled.
  public void testPartialJoinMultiInstantiation() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("joinMultiInstantiation.xpdl");
    String processId = "multiInstantiation";

    Map<String, ProcessDefinition> processMap = getManagementAPI().deployBar(getBarContent(xpdlUrl,
      TestOneJoinActivityInstantiator.class));
    ProcessDefinition process = processMap.get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();
    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);
        
    Set<ActivityInstance<ActivityBody>> actIntances = getQueryRuntimeAPI().getActivityInstances(instanceUUID, "r1");
    assertEquals(2, actIntances.size());
    
    Set<Object> varValues = new HashSet<Object>();
    
    for (ActivityInstance<ActivityBody> activityInstance : actIntances) {
      Map<String, Object> variables = activityInstance.getLastKnownVariableValues();
      assertTrue(variables.containsKey("testVar"));
      varValues.add(variables.get("testVar"));
    }
    
    Set<Object> expectedValues = new HashSet<Object>();
    expectedValues.add("val1");
    expectedValues.add("val2");
    
    assertEquals(expectedValues, varValues);
    
    for (ActivityInstance<ActivityBody> activityInstance : actIntances) {
      ActivityBody body = activityInstance.getBody();
      if (body instanceof TaskInstance) {
        TaskUUID taskUUID = ((TaskInstance) body).getUUID();
        getRuntimeAPI().startTask(taskUUID, false);
        getRuntimeAPI().finishTask(taskUUID, false);
        
        Set<ActivityInstance<ActivityBody>> r2instances = getQueryRuntimeAPI().getActivityInstances(instanceUUID, "r2");
        assertEquals(1, r2instances.size());
      }
    }
    Collection<ActivityInstance<TaskInstance>> tasks = getQueryRuntimeAPI().getTaskList(instanceUUID, TaskState.READY);
    assertEquals(1, tasks.size());
    ActivityInstance<TaskInstance> task = tasks.iterator().next();
    assertEquals(TaskState.READY, task.getBody().getState());
    TaskUUID taskUUID = task.getBody().getUUID();
    getRuntimeAPI().startTask(taskUUID, false);
    getRuntimeAPI().finishTask(taskUUID, false);
    
    checkExecutedOnce(instanceUUID, new String[]{"r2", "BonitaEnd"});
    
    getManagementAPI().undeploy(packageUUID);
  }
}
