package org.ow2.bonita.activity.route;

import java.net.URL;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.activity.instantiation.instantiator.TestOneJoinActivityInstantiator;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.BonitaException;


public class CancelJoinXorTest extends APITestCase {

  // Fail because remaining instances are not canceled.
  public void testPartialJoinMultiInstantiation() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("joinXorCancel.xpdl");
    String processId = "joinXorCancel";

    Map<String, ProcessDefinition> processMap = getManagementAPI().deployBar(getBarContent(xpdlUrl,
      TestOneJoinActivityInstantiator.class));
    ProcessDefinition process = processMap.get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();
    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);
        
    Set<ActivityInstance<ActivityBody>> actIntances = getQueryRuntimeAPI().getActivityInstances(instanceUUID, "r1");
    assertEquals(1, actIntances.size());
    
    
    for (ActivityInstance<ActivityBody> activityInstance : actIntances) {
      ActivityBody body = activityInstance.getBody();
      if (body instanceof TaskInstance) {
        TaskUUID taskUUID = ((TaskInstance) body).getUUID();
        getRuntimeAPI().startTask(taskUUID, false);
        getRuntimeAPI().finishTask(taskUUID, false);
      }
    }
    Collection<ActivityInstance<TaskInstance>> tasks = getQueryRuntimeAPI().getTaskList(instanceUUID, TaskState.READY);
    assertEquals(1, tasks.size());
    ActivityInstance<TaskInstance> task = tasks.iterator().next();
    assertEquals(TaskState.READY, task.getBody().getState());
    TaskUUID taskUUID = task.getBody().getUUID();
    // throws exception: r1 -> bonitaEnd 
    // instance is archived -> task on r2 is ready in history, but it is not executable
    getRuntimeAPI().startTask(taskUUID, false);
    getRuntimeAPI().finishTask(taskUUID, false);
    
    checkExecutedOnce(instanceUUID, new String[]{"r1", "r2", "BonitaEnd"});
    
    getManagementAPI().undeploy(packageUUID);
  }

}
