/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.example;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.novabpm.util.Misc;
import org.ow2.novabpm.util.SimpleCallbackHandler;
import org.ow2.novabpm.util.StandardCallbackHandler;


public abstract class AbstractExample {
  protected LoginContext loginContext;

  public abstract String getProcessId();
  public abstract void afterInstantiate(ProcessInstanceUUID instanceUUID,
      ProcessDefinitionUUID processUUID, RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI,
      QueryDefinitionAPI queryDefinitionAPI) throws BonitaException;

  public void execute(final RuntimeAPI runtimeAPI, final QueryRuntimeAPI queryRuntimeAPI,
      final ManagementAPI managementAPI, final QueryDefinitionAPI queryDefinitionAPI, final String barFile,
      final Map<String, Object> variables) throws BonitaException, IOException {

    final ProcessDefinition process = deploy(managementAPI, barFile).get(getProcessId());
    final PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    Misc.badStateIfNull(packageUUID, "packageUUID is null in process : " + process.getProcessId());
    final ProcessInstanceUUID instanceUUID = instantiate(variables, runtimeAPI, process.getUUID());
    afterInstantiate(instanceUUID, process.getUUID(), runtimeAPI, queryRuntimeAPI,  queryDefinitionAPI);
    managementAPI.undeploy(packageUUID);
    managementAPI.deletePackage(packageUUID);
  }

  public void login() throws LoginException {
    final String loginMode = System.getProperty("login.mode");
    if (loginMode != null && "test".equals(loginMode)) {
      this.loginContext = new LoginContext("Bonita", new SimpleCallbackHandler("john", "bpm"));
    } else {
      this.loginContext = new LoginContext("Bonita", new StandardCallbackHandler());
    }
    this.loginContext.login();
  }

  public void logout() throws LoginException {
    this.loginContext.logout();
  }

  public ProcessInstanceUUID instantiate(final Map<String, Object> variables,
      final RuntimeAPI runtimeAPI, final ProcessDefinitionUUID processUUID) throws BonitaException {
    return runtimeAPI.instantiateProcess(processUUID, variables);
  }

  public Map<String, ProcessDefinition> deploy(final ManagementAPI managementAPI,
                                                  final String barFile) throws BonitaException, IOException {

    Misc.checkArgsNotNull(barFile);
    // This line is here to allow examples to be re executed.
    managementAPI.clearHistory();

    byte[] bar = null;
    try {
      bar = Misc.getAllContentFrom(new URL(barFile));
    } catch (final MalformedURLException mue) {
      try {
        bar = Misc.getAllContentFrom(new File(barFile));
      } catch (final FileNotFoundException fnfe) {
        final InputStream in = this.getClass().getResourceAsStream(barFile);
        if (in == null) {
          throw new BonitaRuntimeException("Can't find: "
              + barFile
              + " neither as an URL, nor as a File nor as a ressource in the following CLASSPATH: "
              + System.getProperty("java.class.path"));
        }
        bar = Misc.getAllContentFrom(in);
      }
    }
    return managementAPI.deployBar(bar);
  }
}


