package org.ow2.bonita.example.aw;

import java.net.URL;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.example.aw.hook.Accept;
import org.ow2.bonita.example.aw.hook.CheckDecision;
import org.ow2.bonita.example.aw.hook.Reject;
import org.ow2.bonita.example.aw.instantiator.ApprovalInstantiator;
import org.ow2.bonita.example.aw.performer.UserPerformerAssign;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition.ProcessState;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;



public class AWMultiInstTest extends APITestCase {
  
  private static final String PROCESS_ID = "multiInstantiation";
  private static final String BAR_FILE = "MultiInstantiation.xpdl";
  
  public void testAWMultiInstReject0() throws Exception {
    execute(0, "Reject");
  }
  
  public void testAWMultiInstReject1() throws Exception {
    execute(1, "Reject");
  }
  public void testAWMultiInstAccept2() throws Exception {
    execute(2, "Accept");
  }
  public void testAWMultiInstAccept3() throws Exception {
    execute(3, "Accept");
  }
  
  protected void execute(int numberOfAccepted, String nodeToCheck) throws Exception {
    URL xpdlUrl = this.getClass().getResource(BAR_FILE);

    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        UserPerformerAssign.class, Accept.class, Reject.class, CheckDecision.class, 
            ApprovalInstantiator.class)).get(PROCESS_ID);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    MultiInstantiationApproval aw = new MultiInstantiationApproval();
    aw.setNumberOfAccepted(numberOfAccepted);
    aw.setLoginContext(loginContext);
    aw.initLogins();
    
    ProcessInstanceUUID instanceUUID = aw.instantiate(null, getRuntimeAPI(), processUUID);
    aw.afterInstantiate(instanceUUID, processUUID, getRuntimeAPI(), getQueryRuntimeAPI(), getQueryDefinitionAPI());
    login();
    checkExecutedOnce(instanceUUID, new String[]{nodeToCheck});

    
    getManagementAPI().undeploy(packageUUID);
    assertEquals(0, getQueryDefinitionAPI().getProcesses(PROCESS_ID, ProcessState.DEPLOYED).size());
    getManagementAPI().deletePackage(packageUUID);
  }
  protected String getLogin() {
    return "john";
  }
  protected String getPassword() {
    return "bpm";
  }
}
