/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.example.carpool;

import java.net.URL;
import java.util.Map;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.example.carpool.hook.Association;
import org.ow2.bonita.example.carpool.hook.CancelPlace;
import org.ow2.bonita.example.carpool.hook.CancelRequestDL;
import org.ow2.bonita.example.carpool.hook.Initial;
import org.ow2.bonita.example.carpool.hook.SendMessage;
import org.ow2.bonita.example.carpool.hook.UpdateRequest;
import org.ow2.bonita.example.carpool.hook.WaitAnswer;
import org.ow2.bonita.example.carpool.hook.WaitAnswerDL;
import org.ow2.bonita.example.carpool.hook.WaitRequest;
import org.ow2.bonita.example.carpool.hook.WaitRequestDL;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.BonitaException;

public class CarpoolTest extends APITestCase {

  public void testCarpoolCancelRequest() throws BonitaException {

    ProcessInstance processInstance = instantiate(Carpool.CANCEL_REQUEST);

    checkNotExecuted(processInstance, new String[]{"CancelPlace", "Association"});
    checkExecutedOnce(processInstance,
        new String[]{"Initial", "InitRequest", "CancelRequest", "BonitaEnd"});

    checkExecutedManyTimes(processInstance, new String[]{"WaitRequest"}, 3);
    checkExecutedManyTimes(processInstance, new String[]{"WaitRequestDL"}, 2);
    checkExecutedManyTimes(processInstance, new String[]{"WaitAnswer"}, 2);
    checkExecutedManyTimes(processInstance, new String[]{"WaitAnswerDL"}, 1);

    PackageDefinitionUUID packageUUID = processInstance.getPackageDefinitionUUID();
    getManagementAPI().undeploy(packageUUID);
    getRuntimeAPI().deleteProcessInstance(processInstance.getUUID());
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testCarpoolCancelPlaceImmediately() throws BonitaException {
    ProcessInstance processInstance = instantiate(Carpool.CANCEL_PLACE_IMMEDIATELY);

    checkNotExecuted(processInstance,
        new String[]{"WaitRequestDL", "InitRequest", "WaitAnswer", "Association", "CancelRequest", "WaitAnswerDL"});
    checkExecutedOnce(processInstance, new String[]{"Initial", "WaitRequest", "CancelPlace", "BonitaEnd"});

    PackageDefinitionUUID packageUUID = processInstance.getPackageDefinitionUUID();
    getManagementAPI().undeploy(packageUUID);
    getRuntimeAPI().deleteProcessInstance(processInstance.getUUID());
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testCarpoolCancelPlaceAfter3Wait() throws BonitaException {
    ProcessInstance processInstance = instantiate(Carpool.CANCEL_PLACE_AFTER_3_WAIT);

    checkNotExecuted(processInstance,
        new String[]{"InitRequest", "WaitAnswer", "Association", "CancelRequest", "WaitAnswerDL"});
    checkExecutedOnce(processInstance, new String[]{"Initial", "CancelPlace", "BonitaEnd"});
    checkExecutedManyTimes(processInstance, new String[]{"WaitRequest"}, 4);
    checkExecutedManyTimes(processInstance, new String[]{"WaitRequestDL"}, 3);

    PackageDefinitionUUID packageUUID = processInstance.getPackageDefinitionUUID();
    getManagementAPI().undeploy(packageUUID);
    getRuntimeAPI().deleteProcessInstance(processInstance.getUUID());
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testCarpoolAssociation() throws BonitaException {
    ProcessInstance processInstance = instantiate(Carpool.ASSOCIATION);

    checkNotExecuted(processInstance,
        new String[]{"CancelRequest", "WaitRequestDL", "CancelPlace", "WaitAnswerDL"});
    checkExecutedOnce(processInstance,
        new String[]{"Initial", "WaitRequest", "InitRequest", "WaitAnswer", "BonitaEnd"});

    PackageDefinitionUUID packageUUID = processInstance.getPackageDefinitionUUID();
    getManagementAPI().undeploy(packageUUID);
    getRuntimeAPI().deleteProcessInstance(processInstance.getUUID());
    getManagementAPI().deletePackage(packageUUID);
  }

  private ProcessInstance instantiate(int mode) throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("carpool.xpdl");
    String processId = "carpool";
    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
            Association.class,
            CancelRequestDL.class,
            CancelPlace.class,
            WaitAnswerDL.class,
            WaitRequestDL.class,
            WaitRequest.class,
            SendMessage.class,
            UpdateRequest.class,
            Initial.class,
            WaitAnswer.class)).get(processId);
    ProcessDefinitionUUID processUUID = process.getUUID();

    Carpool carpool = new Carpool(mode);
    Map<String, Object> variables = carpool.getVariables();
    ProcessInstanceUUID instanceUUID = carpool.instantiate(variables,
        getRuntimeAPI(), processUUID);
    carpool.afterInstantiate(instanceUUID, processUUID, getRuntimeAPI(), getQueryRuntimeAPI(), getQueryDefinitionAPI());

    return getQueryRuntimeAPI().getProcessInstance(instanceUUID);
  }

}
