/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.integration.cycle;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.internal.model.NodeImpl;
import org.ow2.bonita.APITestCase;
import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.definition.activity.AbstractActivity;
import org.ow2.bonita.definition.activity.AbstractActivity.IterationDescriptor;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.EnvTool;
import org.ow2.novabpm.util.ClassDataTool;
import org.ow2.novabpm.util.Command;


public class CycleDetectionTest extends APITestCase {

  private static class CycleDetectionCommand implements Command<Object> {

    private final ProcessDefinitionUUID processUUID;

    public CycleDetectionCommand(final ProcessDefinitionUUID processUUID) {
      this.processUUID = processUUID;
    }

    public Object execute(final Environment environment) throws Exception {
      final XpdlProcess xpdlProcess = EnvTool.getRepository().getXpdlProcess(this.processUUID);
      if (xpdlProcess == null) {
        throw new BonitaRuntimeException("Process is null");
      }

      // A1 is an entry node for the  cycle a1 -> a2 -> a3
      NodeImpl n = xpdlProcess.getNode("a1");
      AbstractActivity a = (AbstractActivity) n.getBehaviour();
      List<IterationDescriptor> iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors == null) {
        throw new BonitaRuntimeException("iterationDescriptors is null");
      }
      // a1 reinit join for transition a2->a3
      List<String> l = new ArrayList<String>();
      l.add("a2");
      checkContains(iterationDescriptors, "a3", l);
      // a1 reinit join for transition a3->a1
      l = new ArrayList<String>();
      l.add("a3");
      checkContains(iterationDescriptors, "a1", l);
      // a1 reinit join for transition a1->a2
      l = new ArrayList<String>();
      l.add("a1");
      checkContains(iterationDescriptors, "a2", l);

      // A2 is not an entry node
      n = xpdlProcess.getNode("a2");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors != null && !iterationDescriptors.isEmpty()) {
        throw new BonitaRuntimeException("iterationDescriptors not empty : " + iterationDescriptors.toString());
      }


      // A3 is an entry node for the  cycle a1 -> a2 -> a3
      n = xpdlProcess.getNode("a3");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors == null) {
        throw new BonitaRuntimeException("iterationDescriptors is null");
      }
      // a3 reinit join for transition a2->a3
      l = new ArrayList<String>();
      l.add("a2");
      checkContains(iterationDescriptors, "a3", l);
      // a3 reinit join for transition a3->a1
      l = new ArrayList<String>();
      l.add("a3");
      checkContains(iterationDescriptors, "a1", l);
      // a3 reinit join for transition a1->a2
      l = new ArrayList<String>();
      l.add("a1");
      checkContains(iterationDescriptors, "a2", l);


      // A4 is not an entry node
      n = xpdlProcess.getNode("a4");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors != null && !iterationDescriptors.isEmpty()) {
        throw new BonitaRuntimeException("iterationDescriptors not empty : " + iterationDescriptors.toString());
      }
      // A5 is not an entry node
      n = xpdlProcess.getNode("a5");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors != null && !iterationDescriptors.isEmpty()) {
        throw new BonitaRuntimeException("iterationDescriptors not empty : " + iterationDescriptors.toString());
      }
      // A6 is not an entry node
      n = xpdlProcess.getNode("a6");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors != null && !iterationDescriptors.isEmpty()) {
        throw new BonitaRuntimeException("iterationDescriptors not empty : " + iterationDescriptors.toString());
      }

      // A7 is an entry node for the  cycle a7 -> a7
      n = xpdlProcess.getNode("a7");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors == null) {
        throw new BonitaRuntimeException("iterationDescriptors is null");
      }
      // a7 reinit join for transition a7->a7
      l = new ArrayList<String>();
      l.add("a7");
      checkContains(iterationDescriptors, "a7", l);


      // A8 is an entry node for the cycle a8 -> a9
      // a8 is an entry node for the cycle a8 -> a9 -> a10
      n = xpdlProcess.getNode("a8");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors == null) {
        throw new BonitaRuntimeException("iterationDescriptors is null");
      }
      // a8 reinit join for transition a8->a9
      l = new ArrayList<String>();
      l.add("a8");
      checkContains(iterationDescriptors, "a9", l);
      // a8 reinit join for transition a9->a8 and a10->a8
      l = new ArrayList<String>();
      l.add("a9");
      l.add("a10");
      checkContains(iterationDescriptors, "a8", l);
      // a8 reinit join for transition a9->a10
      l = new ArrayList<String>();
      l.add("a9");
      checkContains(iterationDescriptors, "a10", l);

      // A9 is not an entry node
      n = xpdlProcess.getNode("a9");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors != null && !iterationDescriptors.isEmpty()) {
        throw new BonitaRuntimeException("iterationDescriptors not empty : " + iterationDescriptors.toString());
      }
      // A10 is not an entry node
      n = xpdlProcess.getNode("a10");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors != null && !iterationDescriptors.isEmpty()) {
        throw new BonitaRuntimeException("iterationDescriptors not empty : " + iterationDescriptors.toString());
      }
      
      // A11 is an entry node for the  cycle a11 -> a12 -> a13
      n = xpdlProcess.getNode("a11");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors == null) {
        throw new BonitaRuntimeException("iterationDescriptors is null");
      }
      // a11 reinit join for transition a12->a13
      l = new ArrayList<String>();
      l.add("a12");
      checkContains(iterationDescriptors, "a13", l);
      // a11 reinit join for transition a13->a11
      l = new ArrayList<String>();
      l.add("a13");
      checkContains(iterationDescriptors, "a11", l);
      // a11 reinit join for transition a11->a12
      l = new ArrayList<String>();
      l.add("a11");
      checkContains(iterationDescriptors, "a12", l);

      // A12 is an entry node for the  cycle a12 -> a13 -> a14 and a11 -> a12 -> a13
      n = xpdlProcess.getNode("a12");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors == null) {
        throw new BonitaRuntimeException("iterationDescriptors is null");
      }
      // a12 reinit join for transition a12->a13
      l = new ArrayList<String>();
      l.add("a12");
      checkContains(iterationDescriptors, "a13", l);
      // a12 reinit join for transition a13->a11
      l = new ArrayList<String>();
      l.add("a13");
      checkContains(iterationDescriptors, "a11", l);
      // a12 reinit join for transition a11->a12 a14->a12
      l = new ArrayList<String>();
      l.add("a11");
      l.add("a14");
      checkContains(iterationDescriptors, "a12", l);

      // A13 is not an entry node
      n = xpdlProcess.getNode("a13");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors != null && !iterationDescriptors.isEmpty()) {
        throw new BonitaRuntimeException("iterationDescriptors not empty : " + iterationDescriptors.toString());
      }
      // A14 is not an entry node
      n = xpdlProcess.getNode("a14");
      a = (AbstractActivity) n.getBehaviour();
      iterationDescriptors = a.getIterationDescriptors();
      if (iterationDescriptors != null && !iterationDescriptors.isEmpty()) {
        throw new BonitaRuntimeException("iterationDescriptors not empty : " + iterationDescriptors.toString());
      }
      return null;
    }

    private static void checkContains(final List<IterationDescriptor> iterations, 
        final String joinNode, final List<String> transitionsFrom) {
      final List<String> remainingTransitions = new ArrayList<String>(transitionsFrom);
      for (final IterationDescriptor it : iterations) {
        if (it.getJoinNodeName().equals(joinNode)) {
          final List<String> trs = new ArrayList<String>(it.getTransitionSourceNodes());
          if (trs.containsAll(remainingTransitions)) {
            trs.removeAll(transitionsFrom);
            if (!trs.isEmpty()) {
              throw new BonitaRuntimeException(trs + " transitions are detected but not specified on join node " 
                  + joinNode + ".");
            }
            return;
          } else if (transitionsFrom.containsAll(trs)) {
            remainingTransitions.removeAll(trs);
            if (remainingTransitions.isEmpty()) {
              trs.removeAll(transitionsFrom);
              if (!trs.isEmpty()) {
                throw new BonitaRuntimeException(trs + " transitions are detected but not specified on join node " 
                    + joinNode + ".");
              }
              return;
            }
          } else {
            trs.removeAll(transitionsFrom);
            throw new BonitaRuntimeException(trs + " transitions are detected but not specified on join node " 
                + joinNode + ".");
          }
        }
      }
      throw new BonitaRuntimeException("for join " + joinNode 
          + ", transitions: " + remainingTransitions + " not in " + iterations);
    }
  }

  public void testSimpleCycle() throws Exception {
    final URL xpdlUrl = this.getClass().getResource("cycleDetection.xpdl");
    final String processId = "cycleDetection";

    final ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        HookBeforeTerminateUpdateVariable.class)).get(processId);
    getManagementAPI().deployClass(ClassDataTool.getClassData(CycleDetectionCommand.class));
    final PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    final ProcessDefinitionUUID processUUID = process.getUUID();

    try {
      getCommandAPI().execute(new CycleDetectionCommand(processUUID));
    } finally {
      getManagementAPI().removeClass(CycleDetectionCommand.class.getName());
      getManagementAPI().undeploy(packageUUID);
      getManagementAPI().deletePackage(packageUUID);
    }
  }

}
