package org.ow2.bonita.participant;

import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.logging.Level;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.definition.VariablePerformerAssign;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.BonitaException;

public class ParticipantTest extends APITestCase {

  /*
   *  test role mapper with type = Custom
   */
  public void testRoleMapperCustom() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("roleMapperCustom.xpdl");
    String processId = "roleMapperCustom";

    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        AdminRoleMapper.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    //get task
    Collection<ActivityInstance< TaskInstance >> taskActivities = getQueryRuntimeAPI().getTaskList(instanceUUID, TaskState.READY);
    assertNotNull(taskActivities);
    assertEquals(1, taskActivities.size());
    for (ActivityInstance< TaskInstance > taskActivity : taskActivities) {
      TaskInstance task = taskActivity.getBody();
      getRuntimeAPI().startTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Starting task: " + task);
      }
      getRuntimeAPI().finishTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Task " + task + " terminated.");
      }
    }
    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  /*
   *  Test role mapper with type = Properties
   */
  public void testRoleMapperProperties() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("roleMapperProperties.xpdl");
    String processId = "roleMapperProperties";

    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    //get task
    Collection<ActivityInstance< TaskInstance >> taskActivities = getQueryRuntimeAPI().getTaskList(instanceUUID, TaskState.READY);
    assertNotNull(taskActivities);
    assertEquals(1, taskActivities.size());
    for (ActivityInstance< TaskInstance > taskActivity : taskActivities) {
      TaskInstance task = taskActivity.getBody();
      getRuntimeAPI().startTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Starting task: " + task);
      }
      getRuntimeAPI().finishTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Task " + task + " terminated.");
      }
    }
    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  /*
   *   Test Performer Assignment with type = Property
   */
  public void testPerformerAssignmentLocalVariable() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("performerAssignmentLocalVariable.xpdl");
    String processId = "performerAssignmentProperty";
    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    //get task
    Collection<ActivityInstance< TaskInstance >> taskActivities = getQueryRuntimeAPI().getTaskList(instanceUUID, TaskState.READY);
    assertNotNull(taskActivities);
    assertEquals(1, taskActivities.size());
    for (ActivityInstance< TaskInstance > taskActivity : taskActivities) {
      TaskInstance task = taskActivity.getBody();
      getRuntimeAPI().startTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Starting task: " + task);
      }
      getRuntimeAPI().finishTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Task " + task + " terminated.");
      }
    }
    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  /*
   *   Test Performer Assignment with type = Callback
   */
  public void testPerformerAssignmentCallback() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("performerAssignmentCallback.xpdl");
    String processId = "performerAssignmentCallback";

    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        AdminPerformerAssign.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    //get task
    Collection<ActivityInstance< TaskInstance >> taskActivities = getQueryRuntimeAPI().getTaskList(instanceUUID, TaskState.READY);
    assertNotNull(taskActivities);
    assertEquals(1, taskActivities.size());
    for (ActivityInstance< TaskInstance > taskActivity : taskActivities) {
      TaskInstance task = taskActivity.getBody();
      getRuntimeAPI().startTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Starting task: " + task);
      }
      getRuntimeAPI().finishTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Task " + task + " terminated.");
      }
    }
    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testPerformerAssignmentGlobalVariable() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("performerAssignmentGlobalVariable.xpdl");
    String processId = "variablePerformerAssign";

    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        VariablePerformerAssign.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    assertTrue(getQueryRuntimeAPI().getTaskList(getLogin(), TaskState.READY).isEmpty());

    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    //get task
    Collection<ActivityInstance< TaskInstance >> taskActivities = getQueryRuntimeAPI().getTaskList(instanceUUID, TaskState.READY);
    assertNotNull(taskActivities);
    assertEquals(1, taskActivities.size());
    for (ActivityInstance< TaskInstance > taskActivity : taskActivities) {
      TaskInstance task = taskActivity.getBody();
      getRuntimeAPI().startTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Starting task: " + task);
      }
      getRuntimeAPI().finishTask(task.getUUID(), true);
      if (LOG.isLoggable(Level.FINE)) {
        LOG.fine("Task " + task + " terminated.");
      }
    }
    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testGetTaskList() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("getTaskList.xpdl");
    String processId = "getTaskList";

    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        VariablePerformerAssign.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    assertTrue(getQueryRuntimeAPI().getTaskList(getLogin(), TaskState.READY).isEmpty());

    //create 3 instances
    ProcessInstanceUUID instance1UUID = getRuntimeAPI().instantiateProcess(processUUID);
    ProcessInstanceUUID instance2UUID = getRuntimeAPI().instantiateProcess(processUUID);
    ProcessInstanceUUID instance3UUID = getRuntimeAPI().instantiateProcess(processUUID);
    
    assertFalse(getQueryRuntimeAPI().getTaskList(getLogin(), TaskState.READY).isEmpty());
    assertEquals(3, getQueryRuntimeAPI().getTaskList(getLogin(), TaskState.READY).size());
    
    //get task uuids
    Collection<ActivityInstance<TaskInstance>> instance1Tasks = getQueryRuntimeAPI().getTaskList(instance1UUID, TaskState.READY); 
    assertEquals(1, instance1Tasks.size());
    TaskInstance task1 = instance1Tasks.iterator().next().getBody();
    TaskUUID task1UUID = task1.getUUID();
    
    Collection<ActivityInstance<TaskInstance>> instance2Tasks = getQueryRuntimeAPI().getTaskList(instance2UUID, TaskState.READY); 
    assertEquals(1, instance2Tasks.size());
    TaskInstance task2 = instance2Tasks.iterator().next().getBody();
    TaskUUID task2UUID = task2.getUUID();
    
    Collection<ActivityInstance<TaskInstance>> instance3Tasks = getQueryRuntimeAPI().getTaskList(instance3UUID, TaskState.READY); 
    assertEquals(1, instance3Tasks.size());
    TaskInstance task3 = instance3Tasks.iterator().next().getBody();
    TaskUUID task3UUID = task3.getUUID();
    
    //assign tasks
    getRuntimeAPI().assignTask(task1UUID, "m1");
    getRuntimeAPI().assignTask(task2UUID, "m1");
    getRuntimeAPI().assignTask(task3UUID, "m2");
    
    //verify tasks are well assigned
    assertEquals("m1", getQueryRuntimeAPI().getTask(task1UUID).getBody().getTaskUser());
    assertEquals("m1", getQueryRuntimeAPI().getTask(task2UUID).getBody().getTaskUser());
    assertEquals("m2", getQueryRuntimeAPI().getTask(task3UUID).getBody().getTaskUser());
    
    //check m1 user has 2 tasks
    Collection<ActivityInstance<TaskInstance>> m1Tasks = getQueryRuntimeAPI().getTaskList("m1", TaskState.READY);
    assertEquals(2, m1Tasks.size());
    Iterator<ActivityInstance<TaskInstance>> it = m1Tasks.iterator();
    boolean task1OK = false;
    boolean task2OK = false;
    while (it.hasNext()) {
      TaskUUID taskUUID = it.next().getBody().getUUID();
      if (taskUUID.equals(task1UUID)) {
        task1OK = true;
      } else if (taskUUID.equals(task2UUID)) {
        task2OK = true;
      } else {
        fail("Wrong taskUUID");
      }
    }
    assertTrue(task1OK && task2OK);
    
    //check m1 user has only 1 task for instance1
    Collection<ActivityInstance<TaskInstance>> m1TasksForInstance1 = 
      getQueryRuntimeAPI().getTaskList(instance1UUID, "m1", TaskState.READY);
    assertEquals(1, m1TasksForInstance1.size());
    assertEquals(task1UUID, m1TasksForInstance1.iterator().next().getBody().getUUID());
    
    //check m1 user has only 1 task for instance2
    Collection<ActivityInstance<TaskInstance>> m1TasksForInstance2 = 
      getQueryRuntimeAPI().getTaskList(instance2UUID, "m1", TaskState.READY);
    assertEquals(1, m1TasksForInstance2.size());
    assertEquals(task2UUID, m1TasksForInstance2.iterator().next().getBody().getUUID());
    
    //check m1 user has 2 tasks
    Collection<ActivityInstance<TaskInstance>> m2Tasks = getQueryRuntimeAPI().getTaskList("m2", TaskState.READY);
    assertEquals(1, m2Tasks.size());
    assertEquals(task3UUID, m2Tasks.iterator().next().getBody().getUUID());
    
    //check m1 user has only 1 task for instance3
    m2Tasks = getQueryRuntimeAPI().getTaskList(instance3UUID, "m2", TaskState.READY);
    assertEquals(1, m2Tasks.size());
    assertEquals(task3UUID, m2Tasks.iterator().next().getBody().getUUID());
    
    //check
    assertTrue(getQueryRuntimeAPI().getTaskList(instance1UUID, "m2", TaskState.READY).isEmpty());
    assertTrue(getQueryRuntimeAPI().getTaskList(instance2UUID, "m2", TaskState.READY).isEmpty());
    assertTrue(getQueryRuntimeAPI().getTaskList(instance3UUID, "m1", TaskState.READY).isEmpty());
    
    getRuntimeAPI().deleteProcessInstance(instance1UUID);
    getRuntimeAPI().deleteProcessInstance(instance2UUID);
    getRuntimeAPI().deleteProcessInstance(instance3UUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
    
  }
  
}
