/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.record;

import java.util.Date;
import java.util.Set;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.internal.cmd.Command;
import org.jbpm.pvm.internal.cmd.CommandService;
import org.ow2.bonita.env.EnvGenerator;
import org.ow2.bonita.env.GlobalEnvironmentFactory;
import org.ow2.bonita.facade.runtime.TaskFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.runtime.impl.TaskFullInstanceImpl;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.persistence.QuerierDbSession;
import org.ow2.bonita.persistence.db.QuerierDbSessionImpl;
import org.ow2.bonita.util.EnvConstants;
import org.ow2.bonita.util.EnvTool;


/**
 * @author Guillaume Porcher
 *
 */
public class DbTaskInstTest extends AbstractTaskInstTest {

  protected CommandService commandService;
  
  @Override
  protected void setUp() throws Exception {
    super.setUp();
    GlobalEnvironmentFactory.setEnvironmentFactory(new EnvGenerator().createEnvironmentFactory());
    commandService = GlobalEnvironmentFactory.getEnvironmentFactory().get(CommandService.class);
  }

  @Override
  protected void tearDown() throws Exception {
    GlobalEnvironmentFactory.getEnvironmentFactory().close();
    GlobalEnvironmentFactory.setEnvironmentFactory(null);
    commandService = null;
    super.tearDown();
  }

  private static QuerierDbSession getQuerierDbSession() {
    return EnvTool.getQuerierDbSession(EnvConstants.CORE_QUERIER_SESSION_NAME);
  }
  private static class SaveTask implements Command<TaskFullInstanceImpl> {
    private TaskFullInstanceImpl task;
    
    public SaveTask(TaskFullInstanceImpl task) {
      this.task = task;
    }

    public TaskFullInstanceImpl execute(Environment environment) throws Exception {
      getQuerierDbSession().save(task);
      return task;
    }
  }
  
  private static class UpdateTask implements Command<TaskFullInstanceImpl> {
    private TaskFullInstanceImpl task;
    
    public UpdateTask(TaskFullInstanceImpl task) {
      this.task = task;
    }

    public TaskFullInstanceImpl execute(Environment environment) throws Exception {
      task = getQuerierDbSession().get(TaskFullInstanceImpl.class, task.getDbid());
      if (task != null) {
        assertNotNull(task.toString());
      }
      return task;
    }
  }
  
  class MergeTask implements Command<TaskFullInstanceImpl> {
    private TaskFullInstanceImpl task;
    
    public MergeTask(TaskFullInstanceImpl task) {
      this.task = task;
    }

    public TaskFullInstanceImpl execute(Environment environment) throws Exception {
      ((QuerierDbSessionImpl)getQuerierDbSession()).getSession().merge(task);
      return task;
    }
  }
  
  private static class RemoveTask implements Command<TaskFullInstanceImpl> {
    private TaskFullInstanceImpl task;
    
    public RemoveTask(TaskFullInstanceImpl task) {
      this.task = task;
    }

    public TaskFullInstanceImpl execute(Environment environment) throws Exception {
      environment.get(QuerierDbSession.class).delete(task);
      return task;
    }
  }
  
  @Override
  protected TaskInstance createTaskRecord(TaskUUID taskUUID, String userId, Set<String> candidates,
      TaskState state, Date createdDate) {

    TaskFullInstanceImpl taskRecord = new TaskFullInstanceImpl(taskUUID,
        state,
        createdDate,
        "user");
    taskRecord.setTaskAssign(state, null, candidates, userId);

    commandService.execute(new SaveTask(taskRecord));
    
    return commandService.execute(new UpdateTask(taskRecord));
  }

  private TaskInstance update(TaskInstance taskInstance) {
    commandService.execute(new MergeTask((TaskFullInstanceImpl) taskInstance));
    return commandService.execute(new UpdateTask((TaskFullInstanceImpl) taskInstance));
  }

  protected void remove(TaskInstance taskInstance) {
    commandService.execute(new RemoveTask((TaskFullInstanceImpl) taskInstance));
  }

  @Override
  protected TaskInstance setEndedDate(TaskInstance taskInstance, Date endedTime) {
    ((TaskFullInstance)taskInstance).setEndedDate(endedTime);
    return update(taskInstance);
  }

  @Override
  protected TaskInstance setStartedBy(TaskInstance taskInstance, String startActor) {
    ((TaskFullInstance)taskInstance).setStartedBy(startActor);
    return update(taskInstance);
  }

  @Override
  protected TaskInstance setEndedBy(TaskInstance taskInstance, String endActor) {
    ((TaskFullInstance)taskInstance).setEndedBy(endActor);
    return update(taskInstance);
  }

  @Override
  protected TaskInstance setStartedDate(TaskInstance taskInstance, Date startedTime) {
    ((TaskFullInstance)taskInstance).setStartedDate(startedTime);
    return update(taskInstance);
  }

}
