/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.record;

import java.util.Set;

import org.jbpm.pvm.env.Environment;
import org.jbpm.pvm.internal.cmd.Command;
import org.jbpm.pvm.internal.cmd.CommandService;
import org.ow2.bonita.APITestCase;
import org.ow2.bonita.env.GlobalEnvironmentFactory;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.uuid.IdFactory;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.runtime.XpdlInstance;
import org.ow2.bonita.services.Querier;
import org.ow2.bonita.services.Recorder;
import org.ow2.bonita.services.impl.QuerierListAccessor;
import org.ow2.bonita.services.info.ProcessInstanceCurrentInfo;

/**
 * @author Pierre Vigneras
 * @date Feb 27, 2008
 */
public class JournalTest extends APITestCase {

  protected CommandService commandService;

  @Override
  public void setUp() throws Exception {
    super.setUp();
    this.commandService = GlobalEnvironmentFactory.getEnvironmentFactory().get(CommandService.class);
  }

  @Override
  public void tearDown() throws Exception {
    this.commandService = null;
    super.tearDown();
  }

  public void testRecordNullProcessInstance() {
    try {
      recordInstanceStarted(null, null);
      fail("Should not accept null arguments!");
    } catch (final IllegalArgumentException iae) {
      // ok
    }
  }

  public void testGetNonExistentProcessInstance() {
    final ProcessFullInstance record = getProcessInstance(IdFactory.getNewInstanceUUID());
    assertNull(record);

    final Set<ProcessFullInstance> records = getProcessInstances(IdFactory.getNewProcessUUID());
    assertNotNull(records);
    assertTrue(records.toString(), records.isEmpty());
  }

  public void testGetProcessInstanceNull() {
    try {
      getProcessInstance(null);
      fail("Should not accept a null processInstanceUUID!");
    } catch (final IllegalArgumentException iae) {
      // ok
    }
    try {
      getProcessInstances(null);
      fail("Should not accept a null processInstanceUUID!");
    } catch (final IllegalArgumentException iae) {
      // ok
    }
  }

  private void recordInstanceStarted(final XpdlInstance instance, final String userId) {
    this.commandService.execute(new RecordInstanceStarted(instance, userId));
  }

  private ProcessFullInstance getProcessInstance(final ProcessInstanceUUID instanceUUID) {
    return this.commandService.execute(new GetProcessInstance(instanceUUID));
  }

  private Set<ProcessFullInstance> getProcessInstances(final ProcessDefinitionUUID processDefUUID) {
    return this.commandService.execute(new GetProcessInstances(processDefUUID));
  }

  // Commands

  private class RecordInstanceStarted implements Command<XpdlInstance> {

    private final XpdlInstance instance;
    private final String userId;

    public RecordInstanceStarted(final XpdlInstance instance, final String userId) {
      this.instance = instance;
      this.userId = userId;
    }

    public XpdlInstance execute(final Environment environment) throws Exception {
      final Recorder recorder = environment.get(Recorder.class);
      recorder.recordInstanceStarted(new ProcessInstanceCurrentInfo(
          this.instance != null ? this.instance.getPackageDefinitionUUID() : null,
          this.instance != null ? this.instance.getProcessDefinitionUUID() : null,
          this.instance != null ? this.instance.getUUID() : null,
          (this.instance != null && this.instance.getParentInstance() != null) 
            ? this.instance.getParentInstance().getUUID() : null,
          this.instance != null ? this.instance.getRootExecution().getVariables() : null,
          this.userId));
      return this.instance;
    }
  }

  private class GetProcessInstance implements Command<ProcessFullInstance> {

    private final ProcessInstanceUUID instanceUUID;

    public GetProcessInstance(final ProcessInstanceUUID instanceUUID) {
      this.instanceUUID = instanceUUID;
    }

    public ProcessFullInstance execute(final Environment environment) throws Exception {
      final Querier journal = environment.get(QuerierListAccessor.class).getJournals();
      return journal.getProcessInstance(this.instanceUUID);
    }
  }

  private class GetProcessInstances implements Command<Set<ProcessFullInstance>> {

    private final ProcessDefinitionUUID processDefUUID;

    public GetProcessInstances(final ProcessDefinitionUUID processDefUUID) {
      this.processDefUUID = processDefUUID;
    }

    public Set<ProcessFullInstance> execute(final Environment environment) throws Exception {
      final Querier journal = environment.get(QuerierListAccessor.class).getJournals();
      return journal.getProcessInstances(this.processDefUUID);
    }
  }
}
