/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.util;

import org.jbpm.pvm.env.Environment;
import org.ow2.bonita.APITestCase;
import org.ow2.bonita.env.EnvGenerator;
import org.ow2.bonita.services.Recorder;
import org.ow2.novabpm.util.Command;


public class EnvToolTest extends APITestCase {

  public void testGetNullParameters() {
    try {
      EnvTool.get(null, null);
      fail("Exception should be thrown when calling get method with null parameters");
    } catch (IllegalArgumentException e) {
      // Test passed
      assertTrue(e.getMessage(), e.getMessage().contains("Some parameters are null in org.ow2.bonita.util.EnvTool.get()"));
    }
  }
  
  public void testGetNullKey() {
    try {
      EnvTool.get(Recorder.class, null);
      fail("Exception should be thrown when calling get method with a null key");
    } catch (IllegalArgumentException e) {
      // Test passed
      assertTrue(e.getMessage(), e.getMessage().contains("Some parameters are null in org.ow2.bonita.util.EnvTool.get()"));
    }
  }
  
  public void testGetNullClass() {
    try {
      EnvTool.get(null, "journal");
      fail("Exception should be thrown when calling get method with a null class");
    } catch (IllegalArgumentException e) {
      // Test passed
      assertTrue(e.getMessage(), e.getMessage().contains("Some parameters are null in org.ow2.bonita.util.EnvTool.get()"));
    }    
  }
  
  public void testGetInvalidObjectName() throws Exception {
    getCommandAPI().execute(new Command<Object>() {
      public Object execute(Environment environment) throws Exception {
        try {
          EnvTool.get(Recorder.class, "nonCorrespondingName");
          fail("Exception should be thrown when calling get method with a non corresponding object name");
        } catch (IllegalStateException e) {
          // Test passed
          assertTrue(e.getMessage(), e.getMessage().contains("nonCorrespondingName"));
        }
        return null;
      }
    });
  }
  
  public void testGetObject() throws Exception {
    getCommandAPI().execute(new Command<Object>() {
      public Object execute(Environment environment) throws Exception {
        Recorder recorder = EnvTool.get(Recorder.class, EnvGenerator.JOURNAL_DEFAULT_KEY);
        assertNotNull(recorder);
        return null;
      }
    });
  }
  
  public void testNoEnvironment() {
    try {
      EnvTool.get(Recorder.class, "journal");
      fail("Exception should be thrown when calling get method outside an environment");
    } catch (IllegalStateException e) {
      // Test passed
      assertTrue(e.getMessage(), e.getMessage().contains("Environment is null!"));
    }    
  }
  
  
  
}
